const AWSLambdaCaller = require('./services/AWSLambdaCaller');
const Util = require('./util');
const MediaResolver = require('./util/MediaResolver');
const DialogFlowAccessor = require('./services/DialogFlowAccessor');
const QuoteDataAccessor = require('./services/QuoteDataAccessor');
const KeyMatcherAccessor = require('./services/KeyMatcherAccessor');
const RasaAccessor = require('./services/RasaAccessor');
const defaultFallbackIntent = require('./util/defaultFallbackIntent');
const RequestPreprocessor = require('./util/RequestPreprocessor');
const AsksuiteProcessingNaturalUtils = require('./AsksuiteProcessingNaturalUtils');
const NodeCache = require('node-cache');

const DEFAULT_LANGUAGE = 'pt-br';

const intentsNotFoundCache = new NodeCache( { stdTTL: 3600, checkperiod: 120 } );

class AsksuiteProcessingNatural {
    static get ORDERED_ACCESSORS() {
        return ['DIALOG_FLOW', 'RASA', 'KEY_MATCHER'];
    } // Robot will search for intents in this order default
    static get RASA_FIRST_ORDERED_ACCESSORS() {
        return ['RASA', 'DIALOG_FLOW', 'KEY_MATCHER'];
    } // Robot will search for intents in this order if tryRasaFirst option is set to true

    constructor(config) {
        this.config = config;
        this.utils = new AsksuiteProcessingNaturalUtils(this.config);
    }

    findIntent(request) {

        return new Promise((resolve, reject) => {

            const executor = async (request) => {
                let intent = {
                    fromCache: false,
                };

                request = await RequestPreprocessor.process(request);

                const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
                const dialogFlowAccessor = new DialogFlowAccessor(awsLambdaCaller);
                const keyMatcherAccessor = new KeyMatcherAccessor(awsLambdaCaller);
                const rasaAccessor = new RasaAccessor(request.config.RASA_URL, request.config.RASA_CONFIDENCE_THRESHOLD);

                let orderOption = AsksuiteProcessingNatural.ORDERED_ACCESSORS;
                if (request.tryRasaFirst) {
                    orderOption = AsksuiteProcessingNatural.RASA_FIRST_ORDERED_ACCESSORS;
                }

                request.findIntent = defaultFallbackIntent(request.companyId);
                request.translated = false;
                request.translation = null;

                for (const accessorName of orderOption) {
                    switch (accessorName) {
                    case 'DIALOG_FLOW':
                        intent = await this.utils.findIntentDialogFlow(dialogFlowAccessor, request, false);

                        if (Util.isNaoentendi(intent.intent) && this.utils.resolveLanguage(request.language) !== DEFAULT_LANGUAGE) {
                            intent = await this.utils.findIntentDialogFlow(dialogFlowAccessor, request, true);
                        }

                        break;
                    case 'KEY_MATCHER':
                        intent = await keyMatcherAccessor.resolveText(request);

                        if (Util.isNaoentendi(intent.intent) && this.utils.resolveLanguage(request.language) !== DEFAULT_LANGUAGE) {
                            const toSend = await this.utils.copyRequestToDefaultLanguage(request);
                            intent = await keyMatcherAccessor.resolveText(toSend);
                        }

                        intent.keywordTry = {try: true};
                        break;
                    case 'RASA':
                        intent = await rasaAccessor.resolveText(request, request.needsRasaEntityConfident);
                        break;
                    }

                    request.findIntent = intent;

                    if (intent.intent && !Util.isNaoentendi(intent.intent)) {
                        intent.resolver = accessorName;

                        if (intent.resolver === 'KEY_MATCHER') {
                            intent.keywordTry.match = intent.intent;
                            intent.keywordTry.intent = intent.intent;
                        }

                        //FOR exit condition
                        break;
                    }

                }

                const language = this.utils.resolveLanguage(request.language);
                const key = `${request.companyId}.${request.userId}.${language}`;

                if (Util.isNaoentendi(intent.intent)) {
                    let count = intentsNotFoundCache.get(key) || 0;
                    intentsNotFoundCache.set(key, ++count);
                  
                    const { ATTEMPTS_BEFORE_DETECTING_LANGUAGE, LANGUAGE_DETECTION_CONFIDENCE } = request.config;

                    if (count >= ATTEMPTS_BEFORE_DETECTING_LANGUAGE) {
                        const detection = await this.utils.detectLanguage(request);

                        if (detection.confidence >= LANGUAGE_DETECTION_CONFIDENCE) {
                            const languageCode = detection.isSupported
                              ? detection.language.code
                              : detection.fallbackLanguage

                            if (!request.language.includes(languageCode)) {
                                intent.languageDetected = detection;
                                intentsNotFoundCache.del(key);
                            }
                        }
                    }
                } else {
                    intentsNotFoundCache.del(key);
                }

                intent.translated = request.translated;
                intent.translation = request.translation;

                return intent;

            };

            executor(request).then(resolve).catch(reject);

        });

    }

    resolveMedia(request) {
        return new Promise((resolve, reject) => {
            try {
                const mediaResolver = new MediaResolver(request.companyId);
                mediaResolver.resolve(request.media, request.intents).then(resolve).catch(reject);
            } catch (e) {
                reject(e);
            }
        });
    }

    async extractQuoteData(request) {
        request = await RequestPreprocessor.process(request);

        const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
        const quoteDataAccessor = new QuoteDataAccessor(awsLambdaCaller);

        return await quoteDataAccessor.resolveText(request);
    }

    async extractTextNumbers(request) {
        request = await RequestPreprocessor.process(request);

        const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
        const quoteDataAccessor = new QuoteDataAccessor(awsLambdaCaller);

        return await quoteDataAccessor.resolveTextNumbers(request);
    }

    async findRasaIntent(request) {
        let intent = {};

        intent = defaultFallbackIntent(request.companyId);
        request.findIntent = intent;

        request = await this.utils.copyRequestToDefaultLanguage(request);
        const rasaAccessor = new RasaAccessor(request.config.RASA_URL, request.config.RASA_CONFIDENCE_THRESHOLD);

        return await rasaAccessor.resolveText(request, request.needsRasaEntityConfident);
    }
}

module.exports = AsksuiteProcessingNatural;
